@@ -7,8 +7,14 @@ from mongoengine.common import _import_class
 from mongoengine.connection import DEFAULT_CONNECTION_NAME, get_db
 from mongoengine.pymongo_support import count_documents
 
-__all__ = ('switch_db', 'switch_collection', 'no_dereference',
-           'no_sub_classes', 'query_counter', 'set_write_concern')
+__all__ = (
+    "switch_db",
+    "switch_collection",
+    "no_dereference",
+    "no_sub_classes",
+    "query_counter",
+    "set_write_concern",
+)
 
 
 class switch_db(object):
@@ -38,17 +44,17 @@ class switch_db(object):
         self.cls = cls
         self.collection = cls._get_collection()
         self.db_alias = db_alias
-        self.ori_db_alias = cls._meta.get('db_alias', DEFAULT_CONNECTION_NAME)
+        self.ori_db_alias = cls._meta.get("db_alias", DEFAULT_CONNECTION_NAME)
 
     def __enter__(self):
         """Change the db_alias and clear the cached collection."""
-        self.cls._meta['db_alias'] = self.db_alias
+        self.cls._meta["db_alias"] = self.db_alias
         self.cls._collection = None
         return self.cls
 
     def __exit__(self, t, value, traceback):
         """Reset the db_alias and collection."""
-        self.cls._meta['db_alias'] = self.ori_db_alias
+        self.cls._meta["db_alias"] = self.ori_db_alias
         self.cls._collection = self.collection
 
 
@@ -111,14 +117,15 @@ class no_dereference(object):
         """
         self.cls = cls
 
-        ReferenceField = _import_class('ReferenceField')
-        GenericReferenceField = _import_class('GenericReferenceField')
-        ComplexBaseField = _import_class('ComplexBaseField')
+        ReferenceField = _import_class("ReferenceField")
+        GenericReferenceField = _import_class("GenericReferenceField")
+        ComplexBaseField = _import_class("ComplexBaseField")
 
-        self.deref_fields = [k for k, v in iteritems(self.cls._fields)
-                             if isinstance(v, (ReferenceField,
-                                               GenericReferenceField,
-                                               ComplexBaseField))]
+        self.deref_fields = [
+            k
+            for k, v in iteritems(self.cls._fields)
+            if isinstance(v, (ReferenceField, GenericReferenceField, ComplexBaseField))
+        ]
 
     def __enter__(self):
         """Change the objects default and _auto_dereference values."""
@@ -164,7 +171,7 @@ class no_sub_classes(object):
 class query_counter(object):
     """Query_counter context manager to get the number of queries.
     This works by updating the `profiling_level` of the database so that all queries get logged,
-    resetting the db.system.profile collection at the beginnig of the context and counting the new entries.
+    resetting the db.system.profile collection at the beginning of the context and counting the new entries.
 
     This was designed for debugging purpose. In fact it is a global counter so queries issued by other threads/processes
     can interfere with it
@@ -175,20 +182,17 @@ class query_counter(object):
     - Some queries are ignored by default by the counter (killcursors, db.system.indexes)
     """
 
-    def __init__(self):
+    def __init__(self, alias=DEFAULT_CONNECTION_NAME):
         """Construct the query_counter
         """
-        self.db = get_db()
+        self.db = get_db(alias=alias)
         self.initial_profiling_level = None
-        self._ctx_query_counter = 0             # number of queries issued by the context
+        self._ctx_query_counter = 0  # number of queries issued by the context
 
         self._ignored_query = {
-            'ns':
-                {'$ne': '%s.system.indexes' % self.db.name},
-            'op':                       # MONGODB < 3.2
-                {'$ne': 'killcursors'},
-            'command.killCursors':      # MONGODB >= 3.2
-                {'$exists': False}
+            "ns": {"$ne": "%s.system.indexes" % self.db.name},
+            "op": {"$ne": "killcursors"},  # MONGODB < 3.2
+            "command.killCursors": {"$exists": False},  # MONGODB >= 3.2
         }
 
     def _turn_on_profiling(self):
@@ -238,8 +242,13 @@ class query_counter(object):
         and substracting the queries issued by this context. In fact everytime this is called, 1 query is
         issued so we need to balance that
         """
-        count = count_documents(self.db.system.profile, self._ignored_query) - self._ctx_query_counter
-        self._ctx_query_counter += 1    # Account for the query we just issued to gather the information
+        count = (
+            count_documents(self.db.system.profile, self._ignored_query)
+            - self._ctx_query_counter
+        )
+        self._ctx_query_counter += (
+            1  # Account for the query we just issued to gather the information
+        )
         return count
 
 
