@@ -1,11 +1,24 @@
 import six
 
 from mongoengine.errors import OperationError
-from mongoengine.queryset.base import (BaseQuerySet, CASCADE, DENY, DO_NOTHING,
-                                       NULLIFY, PULL)
+from mongoengine.queryset.base import (
+    BaseQuerySet,
+    CASCADE,
+    DENY,
+    DO_NOTHING,
+    NULLIFY,
+    PULL,
+)
 
-__all__ = ('QuerySet', 'QuerySetNoCache', 'DO_NOTHING', 'NULLIFY', 'CASCADE',
-           'DENY', 'PULL')
+__all__ = (
+    "QuerySet",
+    "QuerySetNoCache",
+    "DO_NOTHING",
+    "NULLIFY",
+    "CASCADE",
+    "DENY",
+    "PULL",
+)
 
 # The maximum number of items to display in a QuerySet.__repr__
 REPR_OUTPUT_SIZE = 20
@@ -57,12 +70,12 @@ class QuerySet(BaseQuerySet):
     def __repr__(self):
         """Provide a string representation of the QuerySet"""
         if self._iter:
-            return '.. queryset mid-iteration ..'
+            return ".. queryset mid-iteration .."
 
         self._populate_cache()
-        data = self._result_cache[:REPR_OUTPUT_SIZE + 1]
+        data = self._result_cache[: REPR_OUTPUT_SIZE + 1]
         if len(data) > REPR_OUTPUT_SIZE:
-            data[-1] = '...(remaining elements truncated)...'
+            data[-1] = "...(remaining elements truncated)..."
         return repr(data)
 
     def _iter_results(self):
@@ -89,7 +102,7 @@ class QuerySet(BaseQuerySet):
                 yield self._result_cache[pos]
                 pos += 1
 
-            # Raise StopIteration if we already established there were no more
+            # return if we already established there were no more
             # docs in the db cursor.
             if not self._has_more:
                 return
@@ -115,7 +128,7 @@ class QuerySet(BaseQuerySet):
         # the result cache.
         try:
             for _ in six.moves.range(ITER_CHUNK_SIZE):
-                self._result_cache.append(next(self))
+                self._result_cache.append(six.next(self))
         except StopIteration:
             # Getting this exception means there are no more docs in the
             # db cursor. Set _has_more to False so that we can use that
@@ -143,10 +156,9 @@ class QuerySet(BaseQuerySet):
         .. versionadded:: 0.8.3 Convert to non caching queryset
         """
         if self._result_cache is not None:
-            raise OperationError('QuerySet already cached')
+            raise OperationError("QuerySet already cached")
 
-        return self._clone_into(QuerySetNoCache(self._document,
-                                                self._collection))
+        return self._clone_into(QuerySetNoCache(self._document, self._collection))
 
 
 class QuerySetNoCache(BaseQuerySet):
@@ -165,17 +177,17 @@ class QuerySetNoCache(BaseQuerySet):
         .. versionchanged:: 0.6.13 Now doesnt modify the cursor
         """
         if self._iter:
-            return '.. queryset mid-iteration ..'
+            return ".. queryset mid-iteration .."
 
         data = []
         for _ in six.moves.range(REPR_OUTPUT_SIZE + 1):
             try:
-                data.append(next(self))
+                data.append(six.next(self))
             except StopIteration:
                 break
 
         if len(data) > REPR_OUTPUT_SIZE:
-            data[-1] = '...(remaining elements truncated)...'
+            data[-1] = "...(remaining elements truncated)..."
 
         self.rewind()
         return repr(data)
@@ -186,10 +198,3 @@ class QuerySetNoCache(BaseQuerySet):
             queryset = self.clone()
         queryset.rewind()
         return queryset
-
-
-class QuerySetNoDeRef(QuerySet):
-    """Special no_dereference QuerySet"""
-
-    def __dereference(items, max_depth=1, instance=None, name=None):
-        return items
